using System;
using System.Windows.Forms;
using System.Drawing;
using DockApp;

namespace DockApp
{
	/// <summary>
	/// Simple animated dockapp
	/// </summary>
	public class AnimatedDockApp : DockApp
	{
		public AnimatedDockApp() 
		{
			AboutText = "Animated DockApp Demonstration\nFramework by Nelson Minar <nelson@monkey.org>";
			Text = "Animated DockApp";

			Timer t = new Timer();
			t.Tick += new EventHandler(timerEvent);
			t.Interval = 100;
			t.Enabled = true;
		}

		Random r = new Random();
		double hue = 0f;
		double hueVelocity = 0.001f;
		int frameNum = 0;

		protected void timerEvent(object sender, EventArgs e) 
		{
			hue += hueVelocity;
			if (hue >= 1)
				hue = hue - 1;
			else if (hue < 0)
				hue = hue + 1;

			// A little random walk in hue space
			if (frameNum++ > 100) 
			{
				frameNum = 0;
				if (r.Next(2) == 0)
					hueVelocity = 0.001 + r.NextDouble() * 0.001;
				else
					hueVelocity = -0.001f - r.NextDouble() * 0.001;
				System.Diagnostics.Debug.WriteLine("hue: " + hue + " v: " + hueVelocity);
			}

			offScreenGraphics.Clear(colorFromAhsb(1.0f, (float)hue, 0.7f, 0.5f));

			paintOffScreen();
		}

		/// <summary>
		/// Construct a Color object from the given alpha, hue, saturation, and brightness.
		/// These HSB values aren't the same as those returned from Color.getHue(), etc.
		/// </summary>
		/// <param name="alpha">Alpha value in the range [0, 1.0]</param>
		/// <param name="h">Hue in the range [0, 1.0). Only fractional portion is used</param>
		/// <param name="s">Saturation in the range [0, 1.0]</param>
		/// <param name="b">Brightness in the range [0, 1.0]</param>
		/// <returns>Color object</returns>
		public static Color colorFromAhsb(float alpha, float h, float s, float b) 
		{
			// Algorithm from http://mini.net/tcl/666
			float red, green, blue;
			if (s == 0) 
			{					// pure grey
				red = b;
				green = b;
				blue = b;
			} 
			else 
			{
				h = (h - (float)Math.Floor(h));   // Restrict hue to [0, 1)
				float h6 = h * 6.0f;

				// Interpret hue
				red   = (h6 <= 3) ? (2 - h6) : (h6 - 4);
				green = (h6 <= 2) ? (h6) : ((h6 <= 5) ? (4 - h6) : (h6 - 6));
				blue  = (h6 <= 1) ? (-h6) : ((h6 <= 4) ? (h6 - 2) : (6 - h6));

				// Normalize RGB to [0, 1]
				red = (red < 0) ? 0 : ((red > 1) ? 1 : red);
				green = (green < 0) ? 0 : ((green > 1) ? 1 : green);
				blue = (blue < 0) ? 0 : ((blue > 1) ? 1 : blue);

				// Factor in saturation and brightness
				red = ((red - 1) * s + 1) * b;
				green = ((green - 1) * s + 1) * b;
				blue = ((blue - 1) * s + 1) * b;
			}
			return Color.FromArgb((int)(alpha*255), (int)(red*255), (int)(green*255), (int)(blue*255));
		}

		[STAThread]
		static void Main() 
		{
			Application.Run(new AnimatedDockApp());
		}
	}
}
