using System;
using System.Drawing;
using System.Windows.Forms;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;
using DockApp;

namespace DockApp
{
	/// <summary>
	/// DockApp that listens on a socket
	/// </summary>
	public class TCPDockApp : DockApp
	{
		TcpListener tcpListener;
		Thread listenerThread;
		private int portNumber = 23195;
		public TCPDockApp()
		{
			AboutText = "TCP DockApp\nFramework by Nelson Minar <nelson@monkey.org>";
			Text = "TCP DockApp";

			tcpListener = new TcpListener(portNumber);
			tcpListener.Start();

			displayString("Listening\non port\n" + portNumber);

			listenerThread = new Thread(new ThreadStart(listener));
			listenerThread.Start();

		}

		protected void displayString(String s)
		{
			offScreenGraphics.Clear(Color.FromArgb(0x50, 0x50, 0x30));
			Font font = new Font("Lucida Console", 8);
			SolidBrush brush = new SolidBrush(Color.White);
			offScreenGraphics.DrawString(s, font, brush, -1, 3);
			paintOffScreen();
		}

		protected void listener() 
		{
			try 
			{
				while (true) 
				{
					TcpClient tcpClient = tcpListener.AcceptTcpClient();
					Debug.WriteLine("connected");

					NetworkStream networkStream = tcpClient.GetStream();
					byte [] buffer = new byte[64];

					int bytesRead = networkStream.Read(buffer, 0, buffer.Length);
					displayString(System.Text.Encoding.UTF8.GetString(buffer, 0, bytesRead));

					networkStream.Close();
					tcpClient.Close();
				}
			} catch (SocketException tae) 
			{
				if (tae.ErrorCode == 10004)   // magic code seems to mean an asynchronous Stop().
				{
					// ignore this - we've shut ourselves down
				} 
				else 
				{
					Debug.WriteLine(tae.ErrorCode);
				}
			}
		}

		override protected void closeItem_Click(Object sender, EventArgs e) 
		{
			tcpListener.Stop();
			listenerThread.Interrupt();
			base.closeItem_Click(sender, e);
			Application.Exit();
		}

		[STAThread]
		static void Main() 
		{
			Application.Run(new TCPDockApp());
		}
	}
}
